#include "mmc.h"

// Buffers to store commands and responses
extern int command_buf[9] ; 
extern int response_buf[9];

// Count to keep track of the interrupts
int interrupt_count = 0;

// Flag to indicate that 80 cycles needed for initialization are 
// complete (SPI DMA interrupt)
volatile int done_80_cycles = 0;

// Flag to indicate that data block is received using DMA
volatile int data_block_received = 0;

// Flag to indicate that data block is transmitted using DMA
volatile int data_block_transmitted = 0 ;

// Flags used to indicate that token value (0xFE) is received.
int token_flag = 0;
int token_value = 0;

// Flags used to indicate that data block is written to MMC successfully
int data_response = 0;
volatile int data_response_flag = 0;
volatile int busy = 0x0;

// Command that is being sent to MMC
extern volatile int command ;

// SPI DMA ISR to indicate completion of 80 initialization cycles
void mmc_init_isr(int sig)
{
	done_80_cycles = 1;
	
	// Disable SPI
	*pSPICTL = TXFLSH | RXFLSH ; 
}

// SPI Core Rx ISR.
// SPI is configured for Rx interrupt.
// This ISR is used for sending commands to the card.
void command_isr(int sig)
{
	// Read the Receive buffer and write to Transmit buffer
	if(interrupt_count <= 7)
	{
		response_buf[interrupt_count] = *pRXSPI;
		*pTXSPI = command_buf[interrupt_count+1];
	}
	
	// Read receive buffer for the last time
	if(interrupt_count == 8)
	{
		response_buf[interrupt_count] = *pRXSPI;
	}
		
	// Increment the interrupt counter
	interrupt_count++;
	
	// If the counter is equal to 9, disable SPI
	if(interrupt_count == 9)
	{
		*pSPICTL = TXFLSH | RXFLSH ; 
	}
}

// SPI Core Rx ISR -  Used by mmc_read_data function
// Continuously wait until the MMC card sends the data token (0xFE) 
void wait_for_token(int sig)
{
	token_value = *pRXSPI;

	if(token_value == DATA_TOKEN)
	{
		token_flag = 1;
		
		// Disable SPI
		*pSPICTL = TXFLSH | RXFLSH ; 
	}
	else
		*pTXSPI = 0xff;
}

// SPI DMA Rx ISR
// SPI is configured for Rx DMA interrupt
void read_block_isr(int sig)
{
	int status;
	data_block_received = 1;
	
	// Wait till the DMA is complete
	// and then disable SPI
	status = (*pSPISTAT & (SPIF|RXS)) | (*pSPIDMAC & (SPIDMAS|SPIS0|SPIS1));
	while(status!=0)
	{
		status = (*pSPISTAT & (SPIF|RXS)) | (*pSPIDMAC & (SPIDMAS|SPIS0|SPIS1));
	}
	
	// Disable SPI
	*pSPICTL = TXFLSH | RXFLSH ; 
	*pSPIDMAC = FIFOFLSH;
}

// SPI DMA Tx ISR
// SPI is configured for Tx DMA interrupt
void write_block_isr(int sig)
{
	int status;
	data_block_transmitted = 1;
	
	// Wait till the DMA is complete
	// and then disable SPI
	status = (*pSPISTAT & (SPIF|TXS)) | (*pSPIDMAC & (SPIDMAS|SPIS0|SPIS1));
	while(status!=0)
	{
		status = (*pSPISTAT & (SPIF|TXS)) | (*pSPIDMAC & (SPIDMAS|SPIS0|SPIS1));
	}
	
	// Disable SPI
	*pSPICTL = TXFLSH | RXFLSH ; 
	*pSPIDMAC = FIFOFLSH;
}

// SPI Core Rx ISR - Used by mmc_write_data function
// Continuously wait until the MMC card sends a 0x0 
// followed by a high (meaning data is written successfully) 
void wait_for_data_response(int sig)
{
	data_response = *pRXSPI;

	// Disable SPI if MMC card has finished writing data
	if((busy == 1) && (data_response == 0xff))
	{
		*pSPICTL = TXFLSH | RXFLSH ; 
		data_response_flag = 1;
	}
	else if ((data_response == 0x0) && (busy ==0))
	{
		busy = 1;	
		*pTXSPI = 0xff;
	}
	else
		*pTXSPI = 0xff;
}
